/**
 * @EndungsFilter.java     06.01.2011
 * Copyright 2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent;

import java.io.File;

/**
 * Stellt die Endungsfilter der Dateiauswahl-Dialoge bereit. gefiltert wird nach
 * <ul>
 * <li>html</li>
 * <li>htm</li>
 * <li>xml</li>
 * <li>txt</li>
 * </ul>
 * 
 */
public abstract class EndungsFilter extends javax.swing.filechooser.FileFilter {
	public String getSuffix(File f) {
		String s = f.getPath(), suffix = null;
		int i = s.lastIndexOf('.');

		if (i > 0 && i < s.length() - 1)
			suffix = s.substring(i + 1).toLowerCase();
		return suffix;
	}

	public boolean accept(File f) {
		return f.isDirectory();
	}
}

class TextFilter extends EndungsFilter {
	public boolean accept(File f) {
		boolean accept = super.accept(f);
		String suffix = super.getSuffix(f);

		if (suffix != null) {
			// accept() prueft, ob 'f' ein Verzeichnis ist
			accept = super.accept(f) || suffix.equals("txt");
		}
		return accept;
	}

	public String getDescription() {
		return "Text-Dateien (*.txt)";
	}
}

class XMLFilter extends EndungsFilter {
	public boolean accept(File f) {
		boolean accept = super.accept(f);
		String suffix = super.getSuffix(f);

		if (suffix != null) {
			accept = super.accept(f) || suffix.toLowerCase().equals("xml");
		}
		return accept;
	}

	public String getDescription() {
		return "XML-Dateien (*.xml)";
	}

	public boolean isXML(String suffix) {
		return suffix.toLowerCase().equals("xml");
	}
}

class HTMLFilter extends EndungsFilter {
	public boolean accept(File f) {
		boolean accept = super.accept(f);
		String suffix = super.getSuffix(f);

		if (suffix != null) {
			accept = super.accept(f) || suffix.toLowerCase().equals("htm")
					|| suffix.toLowerCase().equals("html");
		}
		return accept;
	}

	public String getDescription() {
		return "Markup-Dateien (*.htm, *.html)";
	}

	public boolean isHTML(String suffix) {
		return suffix.toLowerCase().equals("html")
				|| suffix.toLowerCase().equals("htm");
	}
}
