/**
 * @SonderzeichenPanel.java     01.09.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

/**
 * stellt ein Panel mit JButtons fuer jedes einzelne Sonderzeichen bereit
 * Anklicken fuegt das gewaehlte Sonderzeichen temporaer der Konvertierungsliste
 * hinzu.
 */
public class SonderzeichenPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	private Config config = Config.getConfig();
	private CentGUI parent;
	private CentEngine centEngine = CentEngine.getCentEngine();

	public SonderzeichenPanel(CentGUI parent) {
		this.parent = parent;
		init();
	}

	/**
	 * setzt die Sonderzeichen-Karte zusammen
	 */
	private void init() {
		JScrollPane scrollPane = new JScrollPane();
		scrollPane
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setViewportView(writeEntityTable());
		scrollPane.setColumnHeaderView(writeButtonPanel());
		this.setLayout(new BorderLayout());
		this.add(scrollPane);
	}

	/**
	 * erzeugt die Tabelle mit den Sonderzeichen-Buttons im Haupt-Bereich der
	 * Sonderzeichen-Karte
	 * 
	 * @return
	 */
	private JPanel writeEntityTable() {

		Hashtable<Character, String> ht = config.getLib();
		int anz = ht.size();
		JPanel tablePanel = new JPanel();
		tablePanel.setLayout(new GridLayout(anz / 10, 100));

		JPanel buttonPanel = null;
		JButton entityButt;
		String s = "";
		Enumeration<Character> keys = ht.keys();

		// Zaehlvariable fuer den Zeilenumbruch
		int i = 0;
		while (keys.hasMoreElements()) {

			Object o = keys.nextElement();
			String htmlEntity = new String((String) ht.get(o));
			String unicodeEntity = "&#"
					+ new Integer((int) ((Character) o).charValue()).toString()
					+ ";";
			s += o;

			buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
			Color bg = Color.WHITE;
			// alle 10 Buttons die Hintergrundfarbe wechseln
			if ((Math.floor(i / 10) % 2) == 0) {
				bg = Config.HIGHLIGHT_COLOR;
			}
			i++;
			buttonPanel.setBackground(bg);
			entityButt = new JButton(s);
			entityButt.setToolTipText(s + " | " + htmlEntity + " | "
					+ unicodeEntity);
			entityButt
					.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			entityButt.addActionListener(this);
			s = "";
			buttonPanel.add(entityButt);
			tablePanel.add(buttonPanel);
		}
		return tablePanel;
	}

	/**
	 * erzeugt das Button-Panel im oberen Bereich der Sonderzeichen-Karte
	 * 
	 * @return
	 */
	private JPanel writeButtonPanel() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEADING));
		panel.setBackground(Color.WHITE);
		return panel;
	}

	public void actionPerformed(ActionEvent e) {

		centEngine.toggleTempChars(((JButton) e.getSource()).getText());
		parent.reloadSonderzeichen();
	}
}
