/**
 * @CentUpdater.java     27.09.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent;

import java.awt.Desktop;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Checks if there is a new Version on the Server at startup
 */
public class Updater {

	private URL urlRemote;

	public Updater() {
		getPathLocal();
	}

	public void checkUpdate() {
		URL url;
		int update;
		try {
			url = new URL(Config.DOWNLOAD_DIR + "version.xml");
			InputStream in = url.openStream();

			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			// CDATA in Text wandeln
			factory.setCoalescing(true);

			Document doc = factory.newDocumentBuilder().parse(in);

			NodeList list = doc.getElementsByTagName("*");
			String vStr = null, dStr = null;
			int len;
			Element element;
			for (int i = 0; i < list.getLength(); i++) {
				element = (Element) list.item(i);
				// System.err.println(element.getNodeName() + " - " +
				// element.getTextContent());
				if (element.getNodeName().equals(Config.getOS() + "version")) {
					vStr = element.getTextContent();
				}
				if (element.getNodeName().equals(Config.getOS() + "file")) {
					dStr = element.getTextContent();
				}
			}
			if (compareVersions(vStr)) {
				System.out.println("neue Version verf\u00FCgbar: " + vStr);
				if (Config.getOS().equals("mac")) {
					update = JOptionPane
							.showConfirmDialog(
									null,
									"Eine neue Version von "
											+ Config.APP_NAME
											+ " ist verf\u00FCgbar\nAktuell installiert: "
											+ Config.APP_VERSION
											+ "\nVerf\u00FCgbar: "
											+ vStr
											+ "\nSoll "
											+ Config.APP_NAME
											+ " installiert und neu gestartet werden?");

					// wenn das Update durchgefuehrt werden soll, dies
					// durchfuehren
					// die gepackte Datei zunaechst in tmp-Verz. schreiben und
					// ins aktuelle
					// Verz. entpacken
					if (update == JOptionPane.YES_OPTION) {

						InputStream fisRemote = null;
						FileOutputStream fos = null;
						try {
							getDownloadURL(Config.DOWNLOAD_DIR + Config.getOS()
									+ "/" + dStr);
							fisRemote = urlRemote.openStream();

							byte[] bufferRemote = new byte[(int) urlRemote
									.getFile().length()];

							// Download in tmp-File schreiben
							File tmpFile = File
									.createTempFile("tmpCent", "zip");
							tmpFile.deleteOnExit();
							fos = new FileOutputStream(tmpFile);
							System.out.println("Datei wird geschrieben");
							while ((len = fisRemote.read(bufferRemote)) != -1) {
								fos.write(bufferRemote, 0, len);
							}
							System.out.println("Download abgeschlossen!");
							System.out.println("Datei entpacken...");
							extractArchive(tmpFile,
									new File(System.getProperty("user.dir")));

							// neue Version starten
							initialize();
							// aktuelle Anwendung beenden
							System.exit(0);

						} catch (FileNotFoundException e) {
							JOptionPane.showMessageDialog(null,
									"Datei nicht vorhanden!");
							return;
						} finally {
							try {
								if (fisRemote != null) {
									fisRemote.close();
								}
								if (fos != null) {
									fos.flush();
									fos.close();
								}
							} catch (IOException e) {
							}
						}
						return;
					}
				} else {
					update = JOptionPane
							.showConfirmDialog(
									null,
									"Eine neue Version von "
											+ Config.APP_NAME
											+ " ist verf\u00FCgbar\nAktuell installiert: "
											+ Config.APP_VERSION
											+ "\nVerf\u00FCgbar: "
											+ vStr
											+ "\nM\u00F6chten Sie auf die Downloadseite geleitet werden?");
					if (update == JOptionPane.YES_OPTION) {
						Desktop desk = Desktop.getDesktop();
						try {
							desk.browse(new URI(
									"http://www.javabeginners.de/Projekte/Applications/"
											+ Config.APP_NAME + ".php"));
						} catch (URISyntaxException e) {
							e.printStackTrace();
						}
					}
				}

			} else {
				System.out.println("Kein Update verf\u00FCgbar.");
			}

		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
			System.err.println("Versionsinformationen sind nicht ermittelbar!");
		}

	}

	/**
	 * Vergleicht die Versionsnummern der lokalen und der auf dem
	 * Download-Server liegenden Dateien
	 * 
	 * @param s
	 *            String, Versionsstring der entfernten Datei
	 * @return true wenn die Versionsnummer der entfernten Datei hoeher ist die
	 *         diejenige der lokalen Datei
	 */
	private boolean compareVersions(String s) {
		if (s == null)
			return false;
		// remote String-Array
		String[] rStr = s.split("\\.");
		// local String-Array
		String[] lStr = Config.APP_VERSION.split("\\.");

		int l = lStr.length >= rStr.length ? rStr.length : lStr.length;
		int lv, rv;
		for (int i = 0; i < l; i++) {
			lv = Integer.parseInt(lStr[i]);
			rv = Integer.parseInt(rStr[i]);
			if (rv == lv)
				continue;
			if (rv < lv)
				return false;
			if (rv > lv)
				return true;
		}
		return false;
	}

	/**
	 * Zip-File entpacken und Dateihierarchie beibehalten
	 * 
	 * @param archive
	 *            Zip-File
	 * @param destDir
	 *            Zieldirectory (tmp)
	 * @throws ZipException
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	private void extractArchive(File archive, File destDir)
			throws ZipException, IOException, FileNotFoundException {
		if (!destDir.exists()) {
			destDir.mkdir();
		}

		ZipFile zipFile = new ZipFile(archive);
		// Archivinhalt auslesen...
		Enumeration<?> entries = zipFile.entries();
		// ...und schreiben
		byte[] buffer = new byte[16384];
		int len;
		while (entries.hasMoreElements()) {
			ZipEntry entry = (ZipEntry) entries.nextElement();

			String entryFileName = entry.getName();
			// Dateihierarchie rekursiv erstellen
			File dir = buildDirectoryHierarchyFor(entryFileName, destDir);
			// wenn Verzeichnis nicht existiert, mit Unterverzeichnissen
			// erstellen
			if (!dir.exists()) {
				dir.mkdirs();
			}
			// wenn kein Verzeichnis Datei schreiben
			if (!entry.isDirectory()) {
				BufferedOutputStream bos = null;
				BufferedInputStream bis = null;

				File outFile = new File(destDir, entryFileName);

				bos = new BufferedOutputStream(new FileOutputStream(outFile));

				bis = new BufferedInputStream(zipFile.getInputStream(entry));

				while ((len = bis.read(buffer)) > 0) {
					bos.write(buffer, 0, len);
				}

				bos.flush();
				bos.close();
				bis.close();

			}
		}
	}

	private File buildDirectoryHierarchyFor(String entryName, File destDir) {
		int lastIndex = entryName.lastIndexOf('/');
		String internalPathToEntry = entryName.substring(0, lastIndex + 1);
		File f = new File(destDir, internalPathToEntry);
		// Dateien beruehren
		f.setLastModified(System.currentTimeMillis());
		return new File(destDir, internalPathToEntry);
	}

	/**
	 * Setzt betriebssystemabhaengig den lokalen Pfad zur aktuell ausgefuehten
	 * Datei
	 */
	private String getPathLocal() {
		String name = System.getProperty("user.dir")
				+ System.getProperty("file.separator") + Config.APP_NAME;
		String pathLocal = "";
		if (Config.getOS().equals("mac"))
			pathLocal = name + ".app";
		if (Config.getOS().equals("win"))
			pathLocal = name + ".exe";
		if (Config.getOS().equals("linux") || Config.getOS().equals("noarch"))
			pathLocal = name + ".jar";
		System.out.println("path local: " + pathLocal);
		return pathLocal;
	}

	/**
	 * setzt den Pfad zur Datei im Downloadverzeichnis des genutztne
	 * Betriebssystems
	 * 
	 * @throws URISyntaxException
	 */
	private void getDownloadURL(String url) {
		try {
			urlRemote = new URL(url);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * initialisiert den Neustart der heruntergeladenen Programmdatei
	 */
	private void initialize() {
		System.out.println("Plattform: " + System.getProperty("os.name")
				+ ", Version: " + System.getProperty("os.version"));
		try {
			if (Config.getOS().equals("mac")) {
				Runtime.getRuntime().exec(
						new String[] {
								"open",
								"-W",
								"-n",
								"-a",
								System.getProperty("user.dir") + "/"
										+ Config.APP_NAME + ".app" });
				System.exit(0);
			}
			if (Config.getOS().equals("win")) {
				String path = System.getProperty("user.dir")
						+ System.getProperty("file.separator")
						+ Config.APP_NAME + ".exe";
				Runtime.getRuntime().exec(path);
			}
			if (Config.getOS().equals("linux")
					|| Config.getOS().equals("noarch")) {
				Runtime.getRuntime().exec(
						new String[] {
								"chmod",
								"+x",
								System.getProperty("user.dir") + "/"
										+ Config.APP_NAME + ".jar" });
				Runtime.getRuntime().exec(
						new String[] {
								"java",
								"-jar",
								System.getProperty("user.dir") + "/"
										+ Config.APP_NAME + ".jar" });
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
