/**
 * @PathSelectorDialog.java     08.11.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 * 
 * Stellt einen Dialog mit einem Textfeld und der Moeglichkeit zur Auswahl eines Verzeichnisses bereit
 * Als Standard wird das Heimatverzeichnis des Users angegeben
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent.components;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class PathSelectorDialog implements ActionListener {

	/**
	 * Beendet des Dialog
	 */
	private JButton okButton;
	
	/**
	 * Oeffnet einen Dateiauswahl-Dialog
	 */
	private JButton selectButton;

	/**
	 * Dieser Dialog
	 */
	private JDialog dialog;
	
	/**
	 * Textfeld zur Anzeige des gewaehlten Pfades
	 */
	private JTextField field;
	
	public PathSelectorDialog(){
		initDialog();
	}

	/**
	 * initiiert das GUI
	 * @return das Panel mit allen Komponenten
	 */
	private JPanel initGUI() {
		okButton = new JButton("ok");
		okButton.addActionListener(this);
		selectButton = new JButton("select");
		selectButton.addActionListener(this);

		field = new JTextField(System.getProperty("user.home"), 35);

		JPanel topPanel = new JPanel(new FlowLayout());
		JPanel bottomPanel = new JPanel(new FlowLayout());
		JPanel mainPanel = new JPanel(new BorderLayout());

		topPanel.add(field);
		topPanel.add(selectButton);
		bottomPanel.add(okButton);
		mainPanel.add(topPanel, BorderLayout.CENTER);
		mainPanel.add(bottomPanel, BorderLayout.SOUTH);
		return mainPanel;
	}

	/**
	 * erzeugt den Dialog
	 */
	private void initDialog() {
		JOptionPane optionPane = new JOptionPane();
		optionPane
				.setMessage("Bitte w\u00E4hlen Sie ein Verzeichnis, in dem die Konfigurationsdateien abgelegt werden sollen.");
		optionPane.setMessageType(JOptionPane.PLAIN_MESSAGE);
		optionPane.setOptions(new Object[] { initGUI() });
		dialog = optionPane.createDialog(null, "Icon/Text Button");
		dialog.setVisible(true);

	}

	/**
	 * Stellt einen FileChooser bereit. Der Pfad des gewaehlten Verzeichnisses
	 * wird bei Betaetigung des 'OK'-Buttons im Textfeld gezeigt 
	 */
	private void selectFile() {
		final JFileChooser chooser = new JFileChooser(
				"Konfigurationsverzeichnis w\u00E4hlen");
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		chooser.setVisible(true);
		final int result = chooser.showOpenDialog(dialog);
		if (result == JFileChooser.APPROVE_OPTION) {
			File f = chooser.getSelectedFile();
			field.setText(f.getAbsolutePath());
		}
		chooser.setVisible(false);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == okButton) {
			dialog.setVisible(false);
			dialog.dispose();
		}
		if (e.getSource() == selectButton) {
			selectFile();
		}
	}
	
	/**
	 * 
	 * @return den Inhalt des Textfeldes
	 */
	public String getPath(){
		return field.getText();
	}
}
