/**
 * @SearchDialog.java     08.08.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent.dialogs;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.text.JTextComponent;

import de.yourwebs.cent.CentEngine;
import de.yourwebs.cent.CentGUI;
import de.yourwebs.cent.Config;
import de.yourwebs.cent.MyHighlightPainter;

/**
 * stellt einen Suchdialog bereit
 * 
 */
public class SearchDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;

	private JButton okButt, replaceButt, closeButt;

	private JLabel searchLabel, replaceLabel, charLabel;

	private JTextField searchField, replaceField;

	private JPanel mainPanel, searchPanel, replacePanel, tfPanel;
	
	private JCheckBox aktivCB;

	private Properties properties = Config.getConfig().getProperties();

	private CentGUI parent = CentGUI.getCentGUI();
	
	private String ersetzenStr;

	public SearchDialog() {
		super();
		
		// in Optionen-Dialog fest eingetragene Strings
		ersetzenStr = CentEngine.getCentEngine().getTauschStr();

		initComponents();

		this.pack();
		this.setAlwaysOnTop(true);
		this.setLocationRelativeTo(parent);
		this.setTitle("suchen/ersetzen");
		this.setVisible(true);
	} // Konstruktor

	/**
	 * Factory-Methode f&uuml;r das GridBagLayout
	 * 
	 * @param c
	 *            die Komponente, um deren Layout es geht
	 * @param constraints
	 *            die Constraints
	 * @param x
	 *            absoluter X-Wert der linken oberen Ecke
	 * @param y
	 *            absoluter Y-Wert der linken oberen Ecke
	 * @param w
	 *            Breite
	 * @param h
	 *            H&ouml;he
	 * @param p
	 *            JPanel auf dem das Layout stattfindet
	 */
	public static void add(Component c, GridBagConstraints constraints, int x,
			int y, int w, int h, JPanel p) {
		constraints.gridx = x;
		constraints.gridy = y;
		constraints.gridwidth = w;
		constraints.gridheight = h;
		p.add(c, constraints);
	}

	/**
	 * 
	 */
	private void initComponents() {

		GridLayout gLayout = new GridLayout(1, 2, 10, 10);
		
		EmptyBorder eb = new EmptyBorder(5,5,5,5);

		searchLabel = new JLabel("suchen:");
		searchField = new JTextField(10);
		searchPanel = new JPanel(gLayout);
		searchPanel.add(searchLabel);
		searchPanel.add(searchField);
		searchPanel.setBorder(eb);

		replaceLabel = new JLabel("ersetzen durch:");
		replaceField = new JTextField(10);
		replacePanel = new JPanel(gLayout);
		replacePanel.add(replaceLabel);
		replacePanel.add(replaceField);
		replacePanel.setBorder(eb);
		
		tfPanel = new JPanel(new GridLayout(2,1));
		tfPanel.setBorder(new LineBorder(Config.BORDER_COLOR, 1));
		tfPanel.add(searchPanel);
		tfPanel.add(replacePanel);
		
		aktivCB = new JCheckBox("Eintr\u00E4ge:", false);
		// nur aktivieren wenn die Tabelle im Optionen-Dialog aktiviert ist
		aktivCB.setEnabled(new Boolean(properties.getProperty("ACTIVATE_TABLE")));
		aktivCB.addActionListener(this);

		// in Optionen-Dialog fest eingetragene Strings anzeigen
		charLabel = new JLabel();

		mainPanel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
		mainPanel.setLayout(gbl);

		Insets insets = new Insets(10, 10, 10, 10);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.insets = insets;

		add(tfPanel, gbc, 1, 1, 2, 1, mainPanel);
		add(aktivCB, gbc, 1, 2, 1, 1, mainPanel);
		add(charLabel, gbc, 2, 2, 1, 1, mainPanel);

		// Buttons
		okButt = new JButton("finden");
		okButt.addActionListener(this);

		replaceButt = new JButton("ersetzen");
		replaceButt.addActionListener(this);

		closeButt = new JButton("schlie\u00DFen");
		closeButt.addActionListener(this);

		JPanel buttPanel = new JPanel(new FlowLayout());
		buttPanel.add(okButt);
		buttPanel.add(replaceButt);
		buttPanel.add(closeButt);

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.add(mainPanel);
		panel.add(buttPanel);

		this.add(panel);
	}

	/**
	 * uebersetzt die eingetragenen Farbwerte in ein Color-Objekt
	 * @return das Color-Objekt der Such-Farbe
	 */
	private Color loadColor() {
		int r = new Integer(properties.getProperty("SEARCH_RED")).intValue();
		int g = new Integer(properties.getProperty("SEARCH_GREEN")).intValue();
		int b = new Integer(properties.getProperty("SEARCH_BLUE")).intValue();
		return new Color(r, g, b);
	}
	
	private void showErsetzenStr(boolean show){
		if(!show){
			charLabel.setText("");
			return;
		}
		charLabel.setText(ersetzenStr);
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == okButt) {

			MyHighlightPainter.removeHighlights((JTextComponent) parent
					.getAktComp());
			if (searchField.getText() != null) {
				CentEngine.getCentEngine().highlightString(parent.getAktComp(), loadColor(),
						searchField.getText());
			}
		}
		if (e.getSource() == replaceButt) {
			Component textComp = parent.getAktComp();
			if (textComp instanceof JTextArea) {
				String s = CentEngine.getCentEngine().replaceStr(((JTextArea) textComp).getText(), searchField.getText(), replaceField.getText(), aktivCB.isSelected());
				((JTextArea) textComp).setText(s);
			}
		}
		if (e.getSource() == closeButt) {
			this.setVisible(false);
			this.dispose();
		}
		
		if(e.getSource() == aktivCB){
			showErsetzenStr(aktivCB.isSelected());
		}
	}
}
