/**
 * @ErsetzenTabelle.java     10.09.2008
 * Copyright 2008-2011 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * Die Datei stellt eine zweispaltige Tabelle bereit. Sie wird im Optionen-Dialog unter 'Ersetzen' dargestellt
 * und nimmt auszutauschende Strings und deren Ersatzstrings auf.
 *
 * This file is a part of Cent
 * Cent is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Cent is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Cent;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.cent.dialogs.options;

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;

import de.yourwebs.cent.Config;

/**
 *
 * 
 */
public class ExchangeTable extends JTable {

	private static final long serialVersionUID = 1L;

	private TableModel tableModel;

	private int[] cursorPosition = { 0, 0 };

	public ExchangeTable() {
		tableModel = new TableModel();
		this.setModel(tableModel);

		this.setGridColor(Color.LIGHT_GRAY);
		this.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		this.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		this.setRowSelectionAllowed(false);
		this.setColumnSelectionAllowed(false);
		this.setCellSelectionEnabled(true);
		this.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				setCursorPosition(e.getX(), e.getY());
			}
		});
	} // Konstruktor

	/**
	 * registriert die Mausposition in einer Tabellenzelle und setzt deren
	 * Spalten- und Zeilenindices fest.
	 * 
	 * @param x
	 *            x-Position der Maus
	 * @param y
	 *            y-Position der Maus
	 */
	private void setCursorPosition(int x, int y) {
		Point p = new Point(x, y);
		cursorPosition[0] = this.rowAtPoint(p);
		cursorPosition[1] = this.columnAtPoint(p);
	}

	/**
	 * gibt die Reihen- und Spaltenindices der aktuellen Cursorposition in der
	 * Tabelle aus
	 * 
	 * @return
	 */
	protected int[] getCursorPosition() {
		return cursorPosition;
	}

	/**
	 * Beendet das Bearbeiten einer Zelle und uebernimmt den aktuellen Wert aus
	 * dem Editor
	 * 
	 */
	protected void stopCellEditing() {
		if (this.isEditing()) {
			TableCellEditor editor = getCellEditor();
			if (editor != null) {
				editor.stopCellEditing();
			}
		}
	}

	/**
	 * Waehlt den Text in einer Zelle aus
	 */
	protected void selectTextInCell() {
		int row = getCursorPosition()[0];
		int col = getCursorPosition()[1];
		if (row < 0 | row >= this.getRowCount())
			return;
		this.setRowSelectionInterval(row, row);
		this.editCellAt(this.getSelectedRow(), this.getSelectedColumn());

		// Selektiert den Inhalt der Zelle
		String s = (String) this.getValueAt(row, col);
		TableCellEditor cedit = this.getCellEditor(row, col);
		Component comp = (Component) cedit.getTableCellEditorComponent(this, s,
				true, row, col);
		if (comp instanceof JTextField) {
			JTextField textField = (JTextField) comp;
			textField.selectAll();
			textField.requestFocusInWindow();
		}
	}

	public TableModel getModel() {
		return tableModel;
	}
}

class TableModel extends DefaultTableModel {

	private static final long serialVersionUID = 1L;

	/**
	 * Vorbelegung der Tabelle
	 */
	private static int ROWS = 0, COLS = 2;

	/**
	 * speichert die Tabellendaten
	 */
	private HashMap<Object, Object> hm;

	/**
	 * Tabellendaten
	 */
	private Object[][] data;

	public TableModel() {

		hm = new HashMap<Object, Object>(ROWS);

		String[] columnNames = { "Original", "Ersatz" };

		loadModel();
		loadData();

		this.setDataVector(data, columnNames);
	} // Konstruktor

	private void loadModel() {
		File dataFile = new File(Config.getConfig().getProperties()
				.getProperty("DATA_FILE_NAME"));
		if (!dataFile.exists())
			return;
		if (dataFile.isFile() && dataFile.canRead()) {
			// serialisierte Hashmap laden
			hm = Config.getConfig().loadData();
			// Reihenzahl anpassen
			if (hm != null)
				ROWS = hm.size();
			// Daten in Array einlesen
			data = new Object[ROWS][COLS];
		}
	}

	/**
	 * laedt die Daten der Hashtable in das Model
	 */
	private void loadData() {
		if (!(hm instanceof HashMap))
			return;
		Iterator<Entry<Object, Object>> it = hm.entrySet().iterator();
		int i = 0;

		while (it.hasNext() & i < ROWS) {
			Map.Entry<Object, Object> entry = (Map.Entry<Object, Object>) it
					.next();

			data[i][0] = entry.getKey();
			data[i][1] = entry.getValue();
			i++;
		}
	}

	/**
	 * liest das Model in die HashMap ein; zentrale Datei zum Abspeichern der
	 * Tabelle; wird im Optionen Dialog durch den ActionListener der Buttons
	 * aufgerufen
	 */
	protected void saveModel() {
		String key, value;
		hm.clear();
		for (int i = 0; i < this.getRowCount(); i++) {
			key = (String) this.getValueAt(i, 0);
			value = (String) this.getValueAt(i, 1);
			if (key != null & value != null)
				hm.put(key, value);
		}
		Config.getConfig().saveData(hm);
	}

	public void removeRow(int i) {
		super.removeRow(i);
	}
}
